/*
 * NB:  This file is machine generated, DO NOT EDIT!
 *
 * Edit and run lib/libvcc/generate.py instead.
 */

/*lint -save -e525 -e539 */

#ifdef VCL_RET_MAC
VCL_RET_MAC(abandon, ABANDON,
	VCL_MET_BACKEND_ERROR |
	VCL_MET_BACKEND_FETCH |
	VCL_MET_BACKEND_REFRESH |
	VCL_MET_BACKEND_RESPONSE
)

VCL_RET_MAC(beresp, BERESP,
	VCL_MET_BACKEND_REFRESH
)

VCL_RET_MAC(deliver, DELIVER,
	VCL_MET_BACKEND_ERROR |
	VCL_MET_BACKEND_RESPONSE |
	VCL_MET_DELIVER |
	VCL_MET_HIT |
	VCL_MET_SYNTH
)

VCL_RET_MAC(error, ERROR,
	VCL_MET_BACKEND_FETCH |
	VCL_MET_BACKEND_REFRESH |
	VCL_MET_BACKEND_RESPONSE
)

VCL_RET_MAC(fail, FAIL,
	VCL_MET_BACKEND_ERROR |
	VCL_MET_BACKEND_FETCH |
	VCL_MET_BACKEND_REFRESH |
	VCL_MET_BACKEND_RESPONSE |
	VCL_MET_DELIVER |
	VCL_MET_HASH |
	VCL_MET_HIT |
	VCL_MET_INIT |
	VCL_MET_MISS |
	VCL_MET_PASS |
	VCL_MET_PIPE |
	VCL_MET_PURGE |
	VCL_MET_RECV |
	VCL_MET_SYNTH
)

VCL_RET_MAC(fetch, FETCH,
	VCL_MET_BACKEND_FETCH |
	VCL_MET_MISS |
	VCL_MET_PASS
)

VCL_RET_MAC(hash, HASH,
	VCL_MET_RECV
)

VCL_RET_MAC(lookup, LOOKUP,
	VCL_MET_HASH
)

VCL_RET_MAC(merge, MERGE,
	VCL_MET_BACKEND_REFRESH
)

VCL_RET_MAC(obj_stale, OBJ_STALE,
	VCL_MET_BACKEND_REFRESH
)

VCL_RET_MAC(ok, OK,
	VCL_MET_FINI |
	VCL_MET_INIT
)

VCL_RET_MAC(pass, PASS,
	VCL_MET_BACKEND_RESPONSE |
	VCL_MET_HIT |
	VCL_MET_MISS |
	VCL_MET_RECV
)

VCL_RET_MAC(pipe, PIPE,
	VCL_MET_PIPE |
	VCL_MET_RECV
)

VCL_RET_MAC(purge, PURGE,
	VCL_MET_RECV
)

VCL_RET_MAC(restart, RESTART,
	VCL_MET_DELIVER |
	VCL_MET_HIT |
	VCL_MET_MISS |
	VCL_MET_PASS |
	VCL_MET_PURGE |
	VCL_MET_RECV |
	VCL_MET_SYNTH
)

VCL_RET_MAC(retry, RETRY,
	VCL_MET_BACKEND_ERROR |
	VCL_MET_BACKEND_REFRESH |
	VCL_MET_BACKEND_RESPONSE
)

VCL_RET_MAC(synth, SYNTH,
	VCL_MET_DELIVER |
	VCL_MET_HIT |
	VCL_MET_MISS |
	VCL_MET_PASS |
	VCL_MET_PIPE |
	VCL_MET_PURGE |
	VCL_MET_RECV
)

VCL_RET_MAC(vcl, VCL,
	VCL_MET_RECV
)

#undef VCL_RET_MAC
#endif

#ifdef VCL_MET_MAC
VCL_MET_MAC(backend_error, BACKEND_ERROR, B, (
	(1U << VCL_RET_ABANDON) |
	(1U << VCL_RET_DELIVER) |
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_RETRY))
)

VCL_MET_MAC(backend_fetch, BACKEND_FETCH, B, (
	(1U << VCL_RET_ABANDON) |
	(1U << VCL_RET_ERROR) |
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_FETCH))
)

VCL_MET_MAC(backend_refresh, BACKEND_REFRESH, B, (
	(1U << VCL_RET_ABANDON) |
	(1U << VCL_RET_BERESP) |
	(1U << VCL_RET_ERROR) |
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_MERGE) |
	(1U << VCL_RET_OBJ_STALE) |
	(1U << VCL_RET_RETRY))
)

VCL_MET_MAC(backend_response, BACKEND_RESPONSE, B, (
	(1U << VCL_RET_ABANDON) |
	(1U << VCL_RET_DELIVER) |
	(1U << VCL_RET_ERROR) |
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_PASS) |
	(1U << VCL_RET_RETRY))
)

VCL_MET_MAC(deliver, DELIVER, C, (
	(1U << VCL_RET_DELIVER) |
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_RESTART) |
	(1U << VCL_RET_SYNTH))
)

VCL_MET_MAC(fini, FINI, H, (
	(1U << VCL_RET_OK))
)

VCL_MET_MAC(hash, HASH, C, (
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_LOOKUP))
)

VCL_MET_MAC(hit, HIT, C, (
	(1U << VCL_RET_DELIVER) |
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_PASS) |
	(1U << VCL_RET_RESTART) |
	(1U << VCL_RET_SYNTH))
)

VCL_MET_MAC(init, INIT, H, (
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_OK))
)

VCL_MET_MAC(miss, MISS, C, (
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_FETCH) |
	(1U << VCL_RET_PASS) |
	(1U << VCL_RET_RESTART) |
	(1U << VCL_RET_SYNTH))
)

VCL_MET_MAC(pass, PASS, C, (
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_FETCH) |
	(1U << VCL_RET_RESTART) |
	(1U << VCL_RET_SYNTH))
)

VCL_MET_MAC(pipe, PIPE, C, (
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_PIPE) |
	(1U << VCL_RET_SYNTH))
)

VCL_MET_MAC(purge, PURGE, C, (
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_RESTART) |
	(1U << VCL_RET_SYNTH))
)

VCL_MET_MAC(recv, RECV, C, (
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_HASH) |
	(1U << VCL_RET_PASS) |
	(1U << VCL_RET_PIPE) |
	(1U << VCL_RET_PURGE) |
	(1U << VCL_RET_RESTART) |
	(1U << VCL_RET_SYNTH) |
	(1U << VCL_RET_VCL))
)

VCL_MET_MAC(synth, SYNTH, C, (
	(1U << VCL_RET_DELIVER) |
	(1U << VCL_RET_FAIL) |
	(1U << VCL_RET_RESTART))
)

#undef VCL_MET_MAC
#endif

/*lint -restore */
